texture gDepthBuffer : DEPTHBUFFER;
matrix gProjectionMainScene : PROJECTION_MAIN_SCENE;

texture depthRT;
texture colorRT < string renderTarget = "yes"; >;
float2 sPixelSize = float2(1/1280, 1/720);
float4 maskColor = float4(1.0f,1.0f,1.0f, 1.0f);
#include "mta-helper.fx"
sampler SamplerDepthRealTime = sampler_state
{
    Texture     = (gDepthBuffer);
    AddressU    = Clamp;
    AddressV    = Clamp;
};
sampler2D Sampler0 = sampler_state
{
    Texture = (gTexture0);
};

sampler2D SamplerDepth = sampler_state
{
    Texture = (depthRT);
    AddressU = Clamp;
    AddressV = Clamp;
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = None;
    SRGBTexture = false;
    MaxMipLevel = 0;
    MipMapLodBias = 0;
};

sampler2D SamplerColor = sampler_state
{
    Texture = (colorRT);
    AddressU = Clamp;
    AddressV = Clamp;
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = None;
    SRGBTexture = false;
    MaxMipLevel = 0;
    MipMapLodBias = 0;
};
struct VSInput
{
    float3 Position : POSITION0;
    float3 Normal : NORMAL0;
    float4 Diffuse : COLOR0;
    float2 TexCoord : TEXCOORD0;
};
struct PSInput
{
    float4 Position : POSITION0;
    float4 Diffuse : COLOR0;
    float2 TexCoord : TEXCOORD0;
    float4 Projection : TEXCOORD1;
    float Depth : TEXCOORD2;
};

PSInput VertexShaderFunction(VSInput VS )
{
    PSInput PS = (PSInput)0;
    float4 worldPos = mul(float4(VS.Position.xyz, 1.0f), gWorld);
    float4 viewPos = mul(worldPos, gView);
    float4 Projection = mul(viewPos, gProjection);

    PS.Position = Projection;
    PS.TexCoord = VS.TexCoord;
    PS.Projection = Projection;
    PS.Depth = Projection.z / Projection.w;
    return PS;
}

struct Pixel
{
    float4 World : COLOR0;      // Render target #0
    float4 Color : COLOR1;      // Render target #1
};

float Linearize(float posZ)
{
    return gProjectionMainScene[3][2] / (posZ - gProjectionMainScene[2][2]);
}

float ColorToUnit24New(in float3 color) {
    const float3 scale = float3(65536.0, 256.0, 1.0) / 65793.0;
    return dot(color, scale);
}
float unpack(float4 color)
{
    float value = 0;
    if (color.r > 0 ) value += color.r*0.05;
    else if (color.g > 0 ) value += color.g*0.2 + 0.05;
    else if (color.b > 0 ) value += color.b*0.75 + 0.25;
    return value;
}
float4 pack(float Depth)
{
    if (Depth < 0.05) return float4(Depth/0.05, 0,0,0);
    else if (Depth < 0.25) return float4(0, (Depth-0.05)/0.2,0,0);
    else if (Depth < 0.5) return float4(0, 0,(Depth-0.25)/0.25,0);
    else return float4(0, 0,0,(Depth-0.5)/0.5);   
}
Pixel PixelShaderFunction(PSInput PS)
{
    Pixel output;
    float2 TexCoordProj = ( PS.Projection.xy / PS.Projection.w ) * float2(0.5f, -0.5f) + 0.5f;
    TexCoordProj += 0.5 * sPixelSize;
    float DepthLinNow = Linearize(PS.Depth )*0.01;

    float4 colorX = tex2D(SamplerColor, PS.TexCoord);
    
    output.World =  0;//float4(1.0, 0.2, 0.1, 1.0); 0;
    float4 texel = tex2D(SamplerDepth, TexCoordProj);
    float LastDepth = ColorToUnit24New(texel);
    float d = unpack(texel);
    float4 c = pack(d);
    // output.Color = float4(1.f, 1.f,1.f,1.f);
    output.Color = maskColor;
    if(DepthLinNow*0.93 > unpack(texel)) output.Color = (float4)0.f; //DepthLinNow*0.93 - значение можно изменять, depthFight со своей же моделькой не происходит из за ZEnable = true, а для других объектов этого достаточно
    return output;
}
technique cam_screen_world_mrt
{
    pass P0
    {
        ZEnable = true;
        ZWriteEnable = false;
        AlphaTestEnable = false;
        DepthBias=-0.0001;
        VertexShader = compile vs_3_0 VertexShaderFunction();
        PixelShader = compile ps_3_0 PixelShaderFunction();
    }
}

// Fallback
technique fallback
{
    pass P0
    {
        // Just draw normally
    }
}
